@@ -1,5 +1,4 @@
 import os, sys
-import markdown2
 import re
 from collections import OrderedDict
 try:
@@ -11,8 +10,15 @@ except Exception:
     pass
 from teedoc import Plugin_Base
 from teedoc import Fake_Logger
+from .parse_metadata import Meta_Parser
 
-
+import mistune
+mistune_version = mistune.__version__.split(".") # 0.8.4, 2.0.0rc1
+mistune_version = int(mistune_version[0]) * 10 + int(mistune_version[1]) # 8, 20
+if mistune_version >= 20:
+    from .renderer_new import MDRenderer, plugins
+else:
+    from .renderer_old import MDRenderer
 
 class Plugin(Plugin_Base):
     name = "teedoc-plugin-markdown-parser"
@@ -21,7 +27,7 @@ class Plugin(Plugin_Base):
         "parse_files": ["md"]
     }
 
-    def __init__(self, config, doc_src_path, site_config, logger = None):
+    def on_init(self, config, doc_src_path, site_config, logger = None):
         '''
             @config a dict object
             @logger teedoc.logger.Logger object
@@ -33,24 +39,12 @@ class Plugin(Plugin_Base):
         self.config.update(config)
         self.logger.i("-- plugin <{}> init".format(self.name))
         self.logger.i("-- plugin <{}> config: {}".format(self.name, self.config))
-        self._extention = {
-            # we not need TOC, but we use this extension to generate ID of headers
-            "toc" : {
-                "depth": config["toc_depth"] if "toc_depth" in config else 3
-            },
-            "metadata" : None,
-            "fenced-code-blocks" : None,
-            "highlightjs-lang" : None,
-            "break-on-newline" : None,
-            "code-friendly" : None,
-            "cuddled-lists" : None,
-            "footnotes" : None,
-            "strike" : None,
-            "spoiler" : None,
-            "tables" : None,
-            "task_list" : None
-        }
-        
+        if mistune_version >= 20:
+            self.md_parser = mistune.create_markdown(renderer=MDRenderer(), plugins=plugins)
+        else:
+            renderer = MDRenderer()
+            self.md_parser = mistune.Markdown(renderer=renderer)
+        self.meta_parser = Meta_Parser()
 
     def on_parse_files(self, files, new_config=None):
         # result, format must be this
@@ -72,31 +66,30 @@ class Plugin(Plugin_Base):
                 with open(file, encoding="utf-8") as f:
                     content = f.read().strip()
                     content = self._update_link(content)
-                    parser = markdown2.Markdown(extras = self._extention)
-                    parser._toc_html = ""
                     try:
-                        html = parser.convert(content)
+                        metadata, content_no_meta = self.meta_parser.parse_meta(content)
+                        html = self.md_parser(content_no_meta)
                     except Exception as e:
                         import io, traceback
                         traceback.print_exc()
                         self.logger.w("parse markdown file {} fail, please check markdown content format".format(file))
                         continue
-                    if "title" in html.metadata:
-                        title = html.metadata["title"]
+                    if "title" in metadata:
+                        title = metadata["title"]
                     else:
                         title = ""
-                    if "keywords" in html.metadata and not html.metadata["keywords"].strip() == "":
-                        keywords = html.metadata["keywords"].split(",")
+                    if "keywords" in metadata and not metadata["keywords"].strip() == "":
+                        keywords = metadata["keywords"].split(",")
                     else:
                         keywords = []
-                    if "tags" in html.metadata and not html.metadata["tags"].strip() == "":
-                        tags = html.metadata["tags"].split(",")
+                    if "tags" in metadata and not metadata["tags"].strip() == "":
+                        tags = metadata["tags"].split(",")
                     else:
                         tags = []
-                    if "desc" in html.metadata:
-                        desc = html.metadata["desc"]
+                    if "desc" in metadata:
+                        desc = metadata["desc"]
                     else:
-                        desc = []
+                        desc = ""
                     result["htmls"][file] = {
                         "title": title,
                         "desc": desc,
@@ -105,7 +98,7 @@ class Plugin(Plugin_Base):
                         "body": html,
                         # "toc": html.toc_html if html.toc_html else "",
                         "toc": "", # just empty, toc generated by js but not python
-                        "metadata": html.metadata,
+                        "metadata": metadata,
                         "raw": content
                     }
             else:
